/****************************************
Garden of Eden Creation Kit
  Task-Specific "Kit" Mini-Library
*****************************************

This library allows you to create little kits, that allow you to put things in
them and combine their contents into new and wonderful things.

Of course, this in no way assumes that the player has skills necessary in a role-playing
game to be able to assemble those items.  You have to manually input that checking
in the part where it says //ADD SKILL CHECKING HERE

In fact, there are lots of things you would want to include, such as limited
capacities for the kits and the like. I've added the comments to where you might
want to do that.

But this makes your job a bit easier, in that you don't have to come up with a
design plan to do it. =)

Included for your reference are two example kits, complete with available materials,
and objs that may be created from them.

****************************************

Now with turf kits, too!  I also optimised a large portion of the process; look at
the examples to see the new improved method of creating the kits.

***************************************/

obj
	var/creatable = 1

obj/kit
	var
		material_type = /obj
		creatable_type = /obj

		//maxcapacity = 15

	verb/put_in(obj/O)
		if(!istype(O,material_type))
			usr << "You can't put \an [O] in [src]."
			return
		else

			//ADD CAPACITY CHECKING HERE IF DESIRED
			O.loc = src

			usr << "You put [O] in [src]."

	verb/create_object()
		set src in usr
		var/object_list[] = GenerateObjectsList(src.creatable_type)
		var/obj/choice = input("Create which object?") in object_list+"cancel"
		if(choice == "cancel")
			return
		else
			var/materials_found[0]
			for(var/material in choice:materials)
				// ^- this sees if all materials required are in the kit or not

				var/found = 0

				for(var/obj/O in src)
					if(istype(O,material) && !materials_found.Find(O))
						materials_found += O
						found = 1
						break

				if(!found) //haven't found a required material, so halt the creation process
					var/obj/temp = new material //make a temporary object to get the name
					usr << "A required material, \a [temp], is not available."

					for(var/obj/O in object_list) //cleanup all the creatable objects
						del(O)
					del(temp) //delete the temporary object
					return

			object_list -= choice

			//ADD SKILL CHECKING HERE IF REQUIRED

			usr << "You make a new [choice.name]."

			//MAKE SURE USR CAN CARRY THE NEW OBJECT FIRST?
			choice.loc = usr

			var/obj/X
			for(X in object_list)
				del(X)
			for(X in materials_found)
				del(X)

	verb/take_from(obj/O in src)
		set src in usr

		usr << "You take [O] from [src]."

		//MAKE SURE USR CAN CARRY THE OBJ FIRST?
		O.loc = usr

	verb/look_in()
		set src in usr
		if(src.contents.len)
			usr << "You see:"
			for(var/obj/O in src)
				usr << "\icon[O] \a [O]"
		else
			usr << "[src] is empty."

turf/kit
	var
		material_type = /obj
		creatable_type = /obj

		items[0]

	verb/put_in(obj/O)
		set src in view(1)
		if(!istype(O,material_type))
			usr << "You can't put \an [O] in [src]."
			return
		else

			//ADD CAPACITY CHECKING HERE IF DESIRED
			src.items += O
			usr << "You put [O] in [src]."

	verb/create_object()
		set src in view(1)
		var/object_list[] = GenerateObjectsList(src.creatable_type)
		var/obj/choice = input("Create which object?") in object_list+"cancel"
		if(choice == "cancel")
			return
		else
			var/materials_found[0]
			for(var/material in choice:materials)
				// ^- this sees if all materials required are in the kit or not

				var/found = 0

				for(var/obj/O in src.items)
					if(istype(O,material) && !materials_found.Find(O))
						materials_found += O
						found = 1
						break

				if(!found) //haven't found a required material, so halt the creation process
					var/obj/temp = new material //make a temporary object to get the name
					usr << "A required material, \a [temp], is not available."

					for(var/obj/O in object_list) //cleanup all the creatable objects
						del(O)
					del(temp) //delete the temporary object
					return

			object_list -= choice

			//ADD SKILL CHECKING HERE IF REQUIRED

			usr << "You make a new [choice.name]."

			//MAKE SURE USR CAN CARRY THE NEW OBJECT FIRST?
			choice.loc = usr

			var/obj/X
			for(X in object_list)
				del(X)
			for(X in materials_found)
				del(X)

	verb/take_from(obj/O in src.items)
		set src in view(1)

		usr << "You take [O] from [src]."

		//MAKE SURE USR CAN CARRY THE OBJ FIRST?
		O.loc = usr

	verb/look_in()
		set src in view(1)
		if(src.items.len)
			usr << "You see:"
			for(var/obj/O in src.items)
				usr << "\icon[O] \a [O]"
		else
			usr << "[src] is empty."


//You shouldn't touch this part:
proc/GenerateObjectsList(creatable_type)
	/*
		Takes a list of types in the format
			/obj/weapon/sword
		and spews out actual existing objects
			"a sword"

		If you set the 'creatable' var to zero
		for an object, it is not available to be created by a kit.
	*/
	var/objects[0]
	for(var/X in typesof(creatable_type)-creatable_type)
		var/obj/Y = new X
		if(!Y:creatable)
			del(Y)
		else
			objects += Y
	return objects

obj/kit/proc/AddItem(type,num=1)
	while(num--)
		new type(src)

obj/proc/AddMaterial(type,num=1)
	while(num--)
		src:materials += type

turf/kit/proc/AddItem(type,num=1)
	while(num--)
		src.items += new type

//This object is necessary if you delete all the examples
obj/_GECK/var/materials[]
//It doesn't actually get created, but prevents the compiler
//from giving a bad var error.


//These examples might conflict with a world... only unzip this
//into its own directory.

/*******************************
How to Create A Kit:

Make a new kit.  Set its icon/icon_state to what is desired.

Make a new obj class for the end products.  Add the line
	var/materials[0]
after it.  Eg. /obj/clothing

Make a new obj class for the materials.  Set its icon/
icon_state to what is desired.  Eg. /obj/materials

Change the 'material_type' of the kit to the obj class of
the materials, and the 'creatable_type' to the obj class of
the items it can create.  Eg.
	material_type = /obj/materials
	creatable_type = /obj/clothing

Define the objects that it can create, setting their icons
and applicable vars as desired. Eg.
	/obj/clothing/shirt
	/obj/clothing/pants
	...

Define the types of materials available.  Eg.
	/obj/materials/cloth
		...
	/obj/materials/thread
		...

***NEW!***
Define the materials used for those objects by adding the
line New(), the line ..(), and then AddMaterial(material,number)
for each material required.  Eg.
	New()
		..()
		AddMaterial(/obj/materials/cloth,3) //add three cloth items
		AddMaterial(/obj/materials/thread) //add one thread item
		...

***NEW!***
If desired, you can add stuff to the kit initially by adding the
line New(), the line ..(), and then AddItem(item,number) beneath
the kit class.  Eg.
	New()
		..()
		AddItem(/obj/materials/cloth,3) //put three cloth items in the kit
		AddItem(/obj/materials/thread) //put one thread item in the kit



It should work now!

Look at the kit examples in GECKDemo.dm to see how it works.
*/