/*****************************************************
Mr. Handy
	Task-Specific "Tinkering" Object combo Library
******************************************************

This lets you take two items, stick them together, and get
a hopefully better object.

It's fairly easy to use; it can be tedious when building
a complex object like the one in the example, but for normal
purposes where you just combine one or two items, it's a cinch.

Just follow the examples at the bottom, which illustrate
how to put together and take apart a Glock-17 handgun.
Cool, huh?

"Mr. Handy -- he's sooooo handy!"

*****************************************************/


obj/verb/disassemble()
	if(!src:pieces || !src:pieces.len)
		usr << "[src] cannot be disassembled."
		return
	usr << "You disassemble [src] for the following parts:"
	for(var/X in src:pieces)
		var/obj/B = new X(usr)
		usr << "- [B]"
	del(src)


obj/combo_part
	var/counterparts[0]
	var/results[0]
	var/pieces[0]

	verb
		combine_with(obj/O in usr)
			set src in usr
			if(src.counterparts.Find(O.type))
				var/type = results[src.counterparts.Find(O.type)]
				var/obj/X = new type(usr)
				usr << "You combine [src] with [O] to create \a [X]."
				del(O)
				del(src)
			else
				usr << "You can't combine [src] with [O]."


obj
	proc/AddPieces(piece1)
		//accepts any number of arguments; is used to determine
		//what you get when you break this object back into its
		//components
		for(var/X in args)
			src:pieces += X

	proc/ComboResults(type,result)
		//when combined with type, it produces and object of result type
		src:counterparts += type
		src:results += result


/******************

How to create combo parts and results:

Make combo parts as needed like you'd make any other object.  Be sure
to make them as subobjects of /obj/combo_part, though -- eg.,
/obj/combo_part/thread, /obj/combo_part/cloth, etc.

Add the line New(), the line ..(), then the line ComboResults(type,results)
for each possibility for the part that you will allow it to combine with.

If desired, allow an item to be broken back into its components by adding
the line New(), the line ..(), then the line AddPieces(piece1,piece2...),
with each desired piece in between the parentheses.

Look at MrHandyDemo.dm for examples.

******************/